# -*- coding: utf-8 -*-
# vim: ts=4 nowrap sw=4 sts=4 et
# 
# Quick Highlight Mode
# ====================
# 
# This plugin provides a faster and easier way to select the current document
# highlighting mode. Just press Ctrl+Shift+H, type the language and press Enter.
# 
# Credits
# -------
# 
# Quick Highlight Mode was originally developed by [Nando Vieira](http://simplesideias.com.br/)
# but since he left this project development leaving some unfixed bugs, it has been touched by
# [Fabio Zendhi Nagao](http://zend.lojcomm.com.br/).
# Rewritten for Gedit 3 by [Peter Bakota](http://www.coderbug.rs/).
# 
# License
# -------
# 
# Copyright (C) 2012 Peter Bakota
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# Changes
# -------
# 
# > @2012-07-22
# > Fixed for gedit 3
#

from gi.repository import GObject,Gedit,Gtk,GtkSource
import os

ui_str = """
<ui>
    <menubar name="MenuBar">
        <menu name="ViewMenu" action="View">
            <menuitem name="QuickHighlightMode" action="QuickHighlightMode"/>
        </menu>
    </menubar>
</ui>
"""

GLADE_FILE = os.path.join(os.path.dirname(__file__), "quickhighlightmode.glade")

class QuickHighlightPlugin(GObject.Object, Gedit.WindowActivatable):
    __gtype_name = "QuickHighlightPlugin"
    window = GObject.property(type=Gedit.Window)
    available_ids = {}
    dialog = None

    def __init__(self):
        GObject.Object.__init__(self)

    def do_activate(self):
        self.language_manager = GtkSource.LanguageManager()
        langs = self.language_manager.get_language_ids()
        self.model = Gtk.ListStore(str)
        for id in langs:
            lang = self.language_manager.get_language(id)
            name = lang.get_name()
            self.available_ids[name.lower()] = id
            self.model.append([name])

        quickhighlight = ('QuickHighlightMode', Gtk.STOCK_SELECT_COLOR, 'Quick Highlight', '<Ctrl><Shift>h',
            'Press Ctrl+Shift+H for quick highlight selection',
            self.on_open)

        action_group = Gtk.ActionGroup(self.__class__.__name__)
        action_group.add_actions([quickhighlight],self.window)

        ui_manager = self.window.get_ui_manager()
        ui_manager.insert_action_group(action_group,-1)
        ui_id = ui_manager.add_ui_from_string(ui_str)

        data = { 'action_group': action_group, 'ui_id': ui_id }
        self.window.set_data(self.__class__.__name__, data)

    def do_deactivate(self):
        data = self.window.get_data(self.__class__.__name__)
        ui_manager = self.window.get_ui_manager()
        ui_manager.remove_ui(data['ui_id'])
        ui_manager.remove_action_group(data['action_group'])
        ui_manager.ensure_update()
        self.window.set_data(self.__class__.__name__,None)

    def do_update_state(self):
        data = self.window.get_data(self.__class__.__name__)
        data['action_group'].set_sensitive(self.window.get_active_document() != None)

    def on_open(self, *args):
        if self.dialog:
            self.dialog.set_focus(True)
            return
        builder = Gtk.Builder()
        builder.add_from_file(GLADE_FILE)
        self.dialog = builder.get_object('quickhighlight_dialog')
        self.dialog.connect('delete_event',self.on_close)
        self.dialog.show_all()

        self.cancel_button = builder.get_object('cancel_button')
        self.cancel_button.connect('clicked', self.on_cancel)

        self.apply_button = builder.get_object('apply_button')
        self.apply_button.connect('clicked', self.on_apply)

        self.combo = builder.get_object('language_list')
        self.combo.set_model(self.model)

        self.completion = builder.get_object('entrycompletion1')
        self.completion.connect('match-selected', self.on_selected)
        self.completion.set_model(self.model)
        self.completion.set_text_column(0)

        self.entry = self.combo.get_children()[0]
        self.entry.set_completion(self.completion)

    def on_close(self, *args):
        self.close_dialog()

    def close_dialog(self):
        self.dialog.destroy()
        self.dialog = None

    def on_cancel(self, *args):
        self.close_dialog()

    def on_apply(self, *args):
        lang = self.entry.get_text()
        self.set_lang(lang)

    def on_selected(self, completion, model, iter):
        lang = model.get_value(iter, 0)
        self.set_lang(lang)

    def set_lang(self,lang):
        view = self.window.get_active_view()
        if view:
            lang = lang.lower()
            if self.available_ids.has_key(lang):
                language = self.language_manager.get_language(self.available_ids[lang])
                view.get_buffer().set_language(language)
                self.close_dialog()
